@@ -1,11 +1,12 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 import datetime
+import operator
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
 from trytond.pyson import If, In, Eval, Get
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 
 class PurchaseRequest(ModelSQL, ModelView):
@@ -52,6 +53,15 @@ class PurchaseRequest(ModelSQL, ModelView):
             'create_request': 'Purchase requests are only created by the system.',
             })
 
+    def init(self, module_name):
+        cursor = Transaction().cursor
+        super(PurchaseRequest, self).init(module_name)
+
+        # Migration from 2.0: empty order point origin is -1 instead of 0
+        cursor.execute('UPDATE "%s" '
+            'SET origin = %%s WHERE origin = %%s' % self._table,
+            ('stock.order_point,-1', 'stock.order_point,0'))
+
     def get_rec_name(self, ids, name):
         if isinstance(ids, (int, long)):
             ids = [ids]
@@ -95,7 +105,7 @@ class PurchaseRequest(ModelSQL, ModelView):
         return res
 
     def origin_get(self):
-        model_obj = self.pool.get('ir.model')
+        model_obj = Pool().get('ir.model')
         res = []
         model_ids = model_obj.search([
             ('model', '=', 'stock.order_point'),
@@ -109,11 +119,12 @@ class PurchaseRequest(ModelSQL, ModelView):
         For each product compute the purchase request that must be
         create today to meet product outputs.
         """
-        order_point_obj = self.pool.get('stock.order_point')
-        purchase_request_obj = self.pool.get('purchase.request')
-        product_obj = self.pool.get('product.product')
-        location_obj = self.pool.get('stock.location')
-        user_obj = self.pool.get('res.user')
+        pool = Pool()
+        order_point_obj = pool.get('stock.order_point')
+        purchase_request_obj = pool.get('purchase.request')
+        product_obj = pool.get('product.product')
+        location_obj = pool.get('stock.location')
+        user_obj = pool.get('res.user')
         company = user_obj.browse(Transaction().user).company
 
         # fetch warehouses:
@@ -147,8 +158,7 @@ class PurchaseRequest(ModelSQL, ModelView):
         for min_date in date2products:
             product_ids = [x[0].id for x in date2products[min_date]]
             with Transaction().set_context(forecast=True,
-                    stock_date_end=min_date or datetime.date.max,
-                    stock_skip_warehouse=True):
+                    stock_date_end=min_date or datetime.date.max):
                 pbl = product_obj.products_by_location(warehouse_ids,
                     product_ids, with_childs=True, skip_zero=False)
             for product, max_date in date2products[min_date]:
@@ -173,7 +183,7 @@ class PurchaseRequest(ModelSQL, ModelView):
         return {}
 
     def create_requests(self, new_requests):
-        request_obj = self.pool.get('purchase.request')
+        request_obj = Pool().get('purchase.request')
 
         for new_req in new_requests:
             if new_req['supply_date'] == datetime.date.max:
@@ -193,9 +203,10 @@ class PurchaseRequest(ModelSQL, ModelView):
         to re-create existing requests.
         """
         # delete purchase request without a purchase line
-        uom_obj = self.pool.get('product.uom')
-        request_obj = self.pool.get('purchase.request')
-        product_supplier_obj = self.pool.get('purchase.product_supplier')
+        pool = Pool()
+        uom_obj = pool.get('product.uom')
+        request_obj = pool.get('purchase.request')
+        product_supplier_obj = pool.get('purchase.product_supplier')
         req_ids = request_obj.search([
             ('purchase_line', '=', False),
             ('origin', 'like', 'stock.order_point,%'),
@@ -237,7 +248,7 @@ class PurchaseRequest(ModelSQL, ModelView):
             i.sort(lambda r,s: cmp(r['supply_date'],s['supply_date']))
 
         # Update new requests to take existing requests into account
-        new_requests.sort(lambda r,s: cmp(r['supply_date'],s['supply_date']))
+        new_requests.sort(key=operator.itemgetter('supply_date'))
         for new_req in new_requests:
             for old_req in existing_req.get((new_req['product'].id,
                                              new_req['warehouse']), []):
@@ -261,8 +272,8 @@ class PurchaseRequest(ModelSQL, ModelView):
         :param product: a BrowseRecord of the Product
         :return: a tuple with the two dates
         """
-        product_supplier_obj = self.pool.get('purchase.product_supplier')
-        date_obj = self.pool.get('ir.date')
+        product_supplier_obj = Pool().get('purchase.product_supplier')
+        date_obj = Pool().get('ir.date')
 
         min_date = None
         max_date = None
@@ -286,26 +297,21 @@ class PurchaseRequest(ModelSQL, ModelView):
 
         return (min_date, max_date)
 
-    def compute_request(self, product, location_id, shortage_date,
-            product_quantity, company, order_point=None):
-        """
-        Return the value of the purchase request which will answer to
-        the needed quantity at the given date. I.e: the latest
-        purchase date, the expected supply date and the prefered
-        supplier.
-        """
-        uom_obj = self.pool.get('product.uom')
-        product_supplier_obj = self.pool.get('purchase.product_supplier')
-        date_obj = self.pool.get('ir.date')
+    def find_best_supplier(self, product, date):
+        '''
+        Return the best supplier and purchase_date for the product.
+        '''
+        pool = Pool()
+        date_obj = pool.get('ir.date')
+        product_supplier_obj = pool.get('purchase.product_supplier')
 
         supplier = None
         timedelta = datetime.timedelta.max
         today = date_obj.today()
-        max_quantity = order_point and order_point.max_quantity or 0.0
         for product_supplier in product.product_suppliers:
             supply_date = product_supplier_obj.compute_supply_date(
                     product_supplier, date=today)[0]
-            sup_timedelta = shortage_date - supply_date
+            sup_timedelta = date - supply_date
             if not supplier:
                 supplier = product_supplier.party
                 timedelta = sup_timedelta
@@ -319,10 +325,26 @@ class PurchaseRequest(ModelSQL, ModelView):
 
         if supplier:
             purchase_date = product_supplier_obj.compute_purchase_date(
-                    product_supplier, shortage_date)
+                    product_supplier, date)
         else:
             purchase_date = today
+        return supplier, purchase_date
 
+    def compute_request(self, product, location_id, shortage_date,
+            product_quantity, company, order_point=None):
+        """
+        Return the value of the purchase request which will answer to
+        the needed quantity at the given date. I.e: the latest
+        purchase date, the expected supply date and the prefered
+        supplier.
+        """
+        pool = Pool()
+        uom_obj = pool.get('product.uom')
+
+        supplier, purchase_date = self.find_best_supplier(product,
+            shortage_date)
+
+        max_quantity = order_point and order_point.max_quantity or 0.0
         quantity = uom_obj.compute_qty(product.default_uom,
                 max_quantity - product_quantity,
                 product.purchase_uom or product.default_uom)
@@ -330,7 +352,7 @@ class PurchaseRequest(ModelSQL, ModelView):
         if order_point:
             origin = 'stock.order_point,%s'%order_point.id
         else:
-            origin = 'stock.order_point,0'
+            origin = 'stock.order_point,-1'
         return {'product': product,
                 'party': supplier and supplier or None,
                 'quantity': quantity,
@@ -363,7 +385,7 @@ class PurchaseRequest(ModelSQL, ModelView):
         :param order_point: a BrowseRecord of the Order Point
         :return: a tuple with the date and the quantity
         """
-        product_obj = self.pool.get('product.product')
+        product_obj = Pool().get('product.product')
 
         res_date = None
         res_qty = None
@@ -400,6 +422,55 @@ class PurchaseRequest(ModelSQL, ModelView):
 PurchaseRequest()
 
 
+class CreatePurchaseRequestInit(ModelView):
+    'Create Purchase Request Init'
+    _name = 'purchase.request.create_purchase_request.init'
+    _description = __doc__
+
+CreatePurchaseRequestInit()
+
+
+class CreatePurchaseRequest(Wizard):
+    'Create Purchase Request'
+    _name = 'purchase.request.create_purchase_request'
+
+    states = {
+        'init': {
+            'result': {
+                'type': 'form',
+                'object': 'purchase.request.create_purchase_request.init',
+                'state': [
+                    ('end', 'Cancel', 'tryton-cancel'),
+                    ('create', 'Create', 'tryton-ok', True),
+                    ],
+                },
+            },
+        'create': {
+            'actions': ['_create_purchase_request'],
+            'result': {
+                'type': 'action',
+                'action': '_open',
+                'state': 'end',
+                },
+            },
+        }
+
+    def _create_purchase_request(self, data):
+        purchase_request_obj = Pool().get('purchase.request')
+        purchase_request_obj.generate_requests()
+        return {}
+
+    def _open(self, data):
+        pool = Pool()
+        model_data_obj = pool.get('ir.model.data')
+        act_window_obj = pool.get('ir.action.act_window')
+        act_window_id = model_data_obj.get_id('stock_supply',
+            'act_purchase_request_form_draft')
+        return act_window_obj.read(act_window_id)
+
+CreatePurchaseRequest()
+
+
 class CreatePurchaseAskTerm(ModelView):
     'Create Purchase Ask Term'
     _name = 'purchase.request.create_purchase.ask_term'
@@ -470,7 +541,7 @@ class CreatePurchase(Wizard):
 
     def _set_default_party(self, data):
 
-        request_obj = self.pool.get('purchase.request')
+        request_obj = Pool().get('purchase.request')
         requests = request_obj.browse(data['ids'])
         for request in requests:
             if request.purchase_line:
@@ -482,7 +553,7 @@ class CreatePurchase(Wizard):
 
     def _set_default_term(self, data):
 
-        request_obj = self.pool.get('purchase.request')
+        request_obj = Pool().get('purchase.request')
         requests = request_obj.browse(data['ids'])
         for request in requests:
             if (not request.party) or request.purchase_line:
@@ -493,12 +564,13 @@ class CreatePurchase(Wizard):
         return {'party': request.party.id,'company': request.company.id}
 
     def _create_purchase(self, data):
-        request_obj = self.pool.get('purchase.request')
-        party_obj = self.pool.get('party.party')
-        purchase_obj = self.pool.get('purchase.purchase')
-        product_obj = self.pool.get('product.product')
-        line_obj = self.pool.get('purchase.line')
-        date_obj = self.pool.get('ir.date')
+        pool = Pool()
+        request_obj = pool.get('purchase.request')
+        party_obj = pool.get('party.party')
+        purchase_obj = pool.get('purchase.purchase')
+        product_obj = pool.get('product.product')
+        line_obj = pool.get('purchase.line')
+        date_obj = pool.get('ir.date')
 
         form = data['form']
         if form.get('product') and form.get('party') and \
@@ -592,9 +664,10 @@ class CreatePurchase(Wizard):
         return res
 
     def compute_purchase_line(self, request):
-        party_obj = self.pool.get('party.party')
-        product_obj = self.pool.get('product.product')
-        tax_rule_obj = self.pool.get('account.tax.rule')
+        pool = Pool()
+        party_obj = pool.get('party.party')
+        product_obj = pool.get('product.product')
+        tax_rule_obj = pool.get('account.tax.rule')
 
         line = {
             'product': request.product.id,
